﻿using Hotline.Settings;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;

namespace Hotline.Api.Filter;

public class LogFilterAlphaAttribute : ActionFilterAttribute
{
    private ILogger<LogFilterAlphaAttribute> _logger;
    public string Name { get; set; }

    public bool Record { get; set; }

    public LogFilterAlphaAttribute(string name = "", bool record = true)
    {
        Name = name;
        Record = record;
    }

    public override async Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate next)
    {
        var repository = context.HttpContext.RequestServices.GetService<ISystemLogRepository>();
        _logger = context.HttpContext.RequestServices.GetService<ILogger<LogFilterAlphaAttribute>>();

        if (Record == false)
        {
            await next();
        }
        else
        {
            var request = context.HttpContext.Request;
            var systemLog = new SystemLog
            {
                Name = this.Name,
            };
            GetMethod(context, systemLog);
            var systemId = await repository.AddAsync(systemLog);

            var resultContext = await next();

            var result = context.Result as ObjectResult;
            if (result != null)
                await repository.UpdateResultAsync(systemId, result);
        }
    }

    private void GetMethod(ActionExecutingContext context, SystemLog systemLog)
    {
        var request = context.HttpContext.Request;
        try
        {
            systemLog.ExecuteUrl = request.Method.ToUpper() + "|" + request.Path;
            if (request.Method.ToUpper() == "GET")
            {
                systemLog.ExecuteParam = request.QueryString.ToString();
            }
            else
            {
                if (request.ContentType?.IndexOf("multipart/form-data", StringComparison.Ordinal) == -1)
                {
                    systemLog.ExecuteParam = context.ActionArguments.Values.FirstOrDefault();
                }
            }
        }
        catch (Exception e)
        {
            _logger.LogError(e, "GetMethod 异常");
        }
    }
}
